/**************************************************************************
* Copyright 2019 Pascal Gloor
* 
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*     http://www.apache.org/licenses/LICENSE-2.0
* 
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
**************************************************************************/

#include <stdio.h>
#include <time.h>

#include "dp_accounting.h"
#include "dp_nfqueue.h"
#include "dp_helpers.h"

static dp_accounting *accountings = NULL;
static time_t dp_accountingtime = 0;

void dp_accounting_add(dp_conf *conf, unsigned char *remoteid, int len) {
	//int i;
	dp_accounting *ac;

	// does the element already exist
	HASH_FIND(hh, accountings, remoteid, len, ac);

	if ( conf->debug ) printf("AC: add item\n");

	// found it, increment the counter
	if ( ac ) {
		if ( conf->debug ) printf("AC: item found, incrementing\n");
		ac->count++;
	}
	// not found, create a new one
	else {
		if ( conf->debug ) printf("AC: item not found, creating\n");
		ac = malloc(sizeof(dp_accounting));

		memcpy(ac->remoteid, remoteid, len);
		ac->len = len;
		ac->count = 1;

		HASH_ADD(hh, accountings, remoteid, len, ac);
	}
}

void dp_accounting_cleanup(dp_conf *conf) {
	dp_accounting *ac, *actmp;

	// is it time to cleanup the list?
	// cleanup every conf->interval seconds 
	if ( dp_accountingtime + conf->interval < time(NULL) ) {
		if ( conf->debug ) printf("cleanup AC\n");
		dp_accountingtime = time(NULL);
		HASH_ITER(hh, accountings, ac, actmp) {
			HASH_DEL(accountings, ac);
			free(ac);
		}
	}
}

int dp_accounting_check(dp_conf *conf, unsigned char *remoteid, int len) {
	dp_accounting *ac;

	HASH_FIND(hh, accountings, remoteid, len, ac);

	if ( conf->debug ) printf("AC Check\n");

	if ( ac ) {
		if(conf->debug) printf("AC Check: found item %i > %i ?\n", ac->count, conf->pktint);

		if ( ac->count > conf->pktint ) {
			if(conf->debug) printf("flood detected!\n");
			return NF_DROP;
		}
	}
	return NF_ACCEPT;
}
