/**************************************************************************
* Copyright 2019 Pascal Gloor
* 
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*     http://www.apache.org/licenses/LICENSE-2.0
* 
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
**************************************************************************/

#include <stdio.h>
#include <stdint.h>
#include <netinet/in.h>

#include "dp_dhcpv6.h"
#include "dp_helpers.h"

static dp_dhcpv6_stats dp_v6_stats;

void dp_dhcpv6_check(dp_conf *conf, unsigned char *pkt, int pktlen, int offset, unsigned char **remoteid, int *remoteidlen) {
	uint8_t msgtype = (uint8_t)pkt[offset];

	dp_dhcpv6_cnt((uint16_t)msgtype);

	if ( conf->debug ) printf("offset=%i\n",offset);

	switch(msgtype) {
		case 12: // RELAY-FORW
		case 13: // RELAY-REPL
			offset += 2 + 16 + 16; // msg-type, hop-count, link-addr, peer-addr
			if ( conf->debug ) printf("this is a relay msg\n");
			break;
		default: // all other msgtypes
			offset += 4; // msg-type, transaction-id
	}

	if ( conf->debug ) printf("offset=%i\n",offset);

	while(offset+4<=pktlen) {
		uint16_t code = ntohs(*(uint16_t*)(pkt+offset));
		uint16_t len  = ntohs(*(uint16_t*)(pkt+offset+2));

		if ( conf->debug ) printf("code %i len %i\n", code, len);

		offset+=4;

		dp_dhcpv6_cnt(code);

		if ( code == 9 ) { // relay message
			if ( conf->debug ) printf("option 9, relay msg\n");
			offset+=4;
			continue;
		}

		if ( code == 1 ) { // Client identifier / DUID
			// make sure there's enough space
			if ( offset + len <= pktlen ) {
				*remoteid = pkt+offset;
				*remoteidlen = len;
				break;
			}
		}
		offset+=len;
	}
}

dp_dhcpv6_stats *dp_dhcpv6_cnt(uint16_t msgcode) {

	if ( msgcode > 0 && msgcode < DP_DHCPV6_CODE_LEN ) {
		dp_v6_stats.relcnt[msgcode]++;
		dp_v6_stats.abscnt[msgcode]++;
	}
	return &dp_v6_stats;
}
