/**************************************************************************
* Copyright 2019 Pascal Gloor
* 
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*     http://www.apache.org/licenses/LICENSE-2.0
* 
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
**************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <linux/limits.h>

#include "dp_events.h"
#include "dp_helpers.h"
#include "dp_accounting.h"
#include "dp_blacklist.h"

void *dp_events(void *data) {
	dp_conf *conf = (dp_conf*)data;

	time_t last_blacklist  = time(NULL);
	time_t last_accounting = time(NULL);
	time_t last_stats      = time(NULL);

	while(1) {
		time_t now = time(NULL);

		if ( last_blacklist + DP_BL_CLEAN_INT <= now ) {
			dp_blacklist_cleanup(conf);
			last_blacklist=now;
		}

		if ( last_accounting + DP_AC_CLEAN_INT <= now ) {
			dp_accounting_cleanup(conf);
			last_accounting=now;
		}

		if ( last_stats + conf->stats_interval <= now ) {
			dp_events_stats(conf);
			last_stats=now;
		}

		usleep(500000); /* 100ms */
	}

	return NULL;
}

void dp_events_stats(dp_conf *conf) {
	char tmpfile[PATH_MAX+4];
	dp_dhcpv4_stats *stats4;
	dp_dhcpv6_stats *stats6;
	dp_blacklist *blacklists, *bl, *tmp;
	FILE *fh;
	int i;
	int first = 1;

	snprintf(tmpfile, sizeof(tmpfile), "%s.tmp", conf->stats_file);

	if ( ( fh = fopen(tmpfile, "w") ) == NULL )
		return;

	stats4 = dp_dhcpv4_cnt(0);

	fprintf(fh,"{ \"dhcpv4\": { ");

	for(i=0; i<DP_DHCPV4_CODE_LEN; i++) {
		if ( stats4->relcnt[i]>0 || stats4->abscnt[i]>0 ) {
			if ( !first ) fprintf(fh, ", ");
			fprintf(fh, "\"%s\": { ", dp_dhcpv4_code[i]);
			fprintf(fh, "\"rel\": %llu, \"abs\": %llu }", stats4->relcnt[i], stats4->abscnt[i]);
			first=0;
		}

		stats4->relcnt[i]=0;
	}

	first=1;

	stats6 = dp_dhcpv6_cnt(0);

	fprintf(fh, "}, \"dhcpv6\" : { ");

	for(i=0; i<DP_DHCPV6_CODE_LEN; i++) {
		if ( stats6->relcnt[i]>0 || stats6->abscnt[i]>0 ) {
			if ( !first ) fprintf(fh, ", ");
			fprintf(fh, "\"%s\": { ", dp_dhcpv6_code[i]);
			fprintf(fh, "\"rel\": %llu, \"abs\": %llu }", stats6->relcnt[i], stats6->abscnt[i]);
			first=0;
		}

		stats6->relcnt[i]=0;
	}


	fprintf(fh, "}, \"blacklist\": [");

	blacklists = dp_blacklist_get();

	first=1;
	HASH_ITER(hh, blacklists, bl, tmp) {
		if ( bl->expire > time(NULL) ) {
			if ( !first ) fprintf(fh, ", ");
			fprintf(fh, "{ \"id\": \"%s\", \"expire\": %i }", dp_printid(bl->remoteid, bl->len), (int)(bl->expire - time(NULL)));
		}
	}

	fprintf(fh, "], \"timestamp\": %i }\n", (int)time(NULL));

	fclose(fh);

	rename(tmpfile, conf->stats_file);

}
