/**************************************************************************
* Copyright 2019 Pascal Gloor
* 
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*     http://www.apache.org/licenses/LICENSE-2.0
* 
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pthread.h>

#include "dp_main.h"
#include "dp_helpers.h"
#include "dp_nfqueue.h"
#include "dp_events.h"

// main function
int main(int argc, char **argv) {
	char *configfile;
	dp_conf conf;
	pthread_t pth_nfqueue, pth_events;

	if ( argc == 2 ) {
		configfile = argv[1];
	}
	else {
		dp_usage(argv[0]);
		return EXIT_FAILURE;
	}

	if ( dp_load_config(&conf, configfile) == NULL )
		return EXIT_FAILURE;

	if ( pthread_create(&pth_nfqueue, NULL, &dp_nfq_start, &conf) != 0 ) {
		fprintf(stderr, "pthread_create error: %s\n", strerror(errno));
		return EXIT_FAILURE;
	}

	if ( pthread_create(&pth_events, NULL, &dp_events, &conf) != 0 ) {
		fprintf(stderr, "pthread_create error: %s\n", strerror(errno));
		return EXIT_FAILURE;
	}

	pthread_join(pth_nfqueue, NULL);
	pthread_join(pth_events,  NULL);


	return 0;
}
